package gssim.gssimstudio.editor.gssim.multiDrag
{
	import com.degrafa.core.utils.ColorKeys;
	
	import gssim.gssimstudio.editor.flare.util.Colors;
	
	import mx.graphics.SolidColor;
	import mx.graphics.SolidColorStroke;
	
	import spark.components.Group;
	import spark.primitives.Rect;
	
	/**
	 * Class for handling selection box on the surface
	 */
	public class SelectionBox extends Group
	{
		private var _selectionRectangle : Rect;
		
		public function SelectionBox()
		{
			_selectionRectangle = new Rect();
			_selectionRectangle.x = 0;
			_selectionRectangle.y = 0;
			_selectionRectangle.width = 0;
			_selectionRectangle.height = 0;
			_selectionRectangle.radiusX = 0;
			var rectangleFill : SolidColor = new SolidColor(ColorKeys.BLACK);
			rectangleFill.alpha = 0.0;
			_selectionRectangle.fill = rectangleFill;
			var rectangleStroke : SolidColorStroke = new SolidColorStroke(ColorKeys.DARKGREEN);
			rectangleStroke.weight = 1;
			_selectionRectangle.stroke = rectangleStroke;
			this.addElement(_selectionRectangle);
		}
		
		/**
		 * Begin selection process
		 */
		public function startSelection(x : Number, y : Number) : void
		{
			_selectionRectangle.x = x;
			_selectionRectangle.y = y;
			_selectionRectangle.width = 0;
			_selectionRectangle.height = 0;
			this.visible = true;
		}

		/**
		 * Update selection box bounds
		 */
		public function update(x : Number, y : Number) : void
		{
			_selectionRectangle.width = x - _selectionRectangle.x;
			_selectionRectangle.height = y - _selectionRectangle.y;
		}

		/**
		 * Finish selection process
		 */
		public function stopSelection() : void 
		{
			_selectionRectangle.width = 0;
			_selectionRectangle.height = 0;
			this.visible = false;
		}

		/**
		 * If the box has width or height < 0 then move the point across the appropriate axis and change sign of property.
		 */
		private function normalize() : void
		{
			if (_selectionRectangle.width < 0)
			{
				_selectionRectangle.x += _selectionRectangle.width;
				_selectionRectangle.width = -_selectionRectangle.width;
			}
			if (_selectionRectangle.height < 0)
			{
				_selectionRectangle.y += _selectionRectangle.height;
				_selectionRectangle.height = -_selectionRectangle.height;
			}
		}

		/**
		 * Returns true if given point (x,y) is in bounds of selection points
		 */
		public function isInBounds(x : Number, y : Number) : Boolean
		{
			normalize();
			if (x > _selectionRectangle.x &&
				x < _selectionRectangle.x + _selectionRectangle.width &&
				y > _selectionRectangle.y &&
				y < _selectionRectangle.y + _selectionRectangle.height)
				return true;
			return false;
		}

	}
}
