package gssim.gssimstudio.editor.gssim.topology
{
	import flash.events.Event;
	import com.degrafa.paint.SolidStroke;
	
	/**
	 * Linker between two GenderObjects in network topology
	 */
	public class NetworkObjectLinker extends Linker
	{
		private var nodeFrom : GenderObject;
		public function get NodeFrom() : GenderObject { return nodeFrom; }
		private var nodeTo : GenderObject;
		public function get NodeTo() : GenderObject { return nodeTo; }
		
		private var _BaudRate : Number;
		
		public function get BaudRate() : Number { return _BaudRate; }
		public function set BaudRate(value : Number) : void
		{
			_BaudRate = value;
			UsedBaudRate = UsedBaudRate;
		}
		
		public var PropDelay : Number;
		
		public var MTU : Number;
		
		private var _UsedBaudRate : Number;
		
		public function get UsedBaudRate() : Number { return _UsedBaudRate; }
		public function set UsedBaudRate(value : Number) : void
		{
			if (isNaN(value)) _UsedBaudRate = 0;
			else if (value == Number.NEGATIVE_INFINITY || value == Number.POSITIVE_INFINITY) _UsedBaudRate = 0;
			else if (value > BaudRate) _UsedBaudRate = BaudRate;
			else if (value < 0) _UsedBaudRate = 0;
			else _UsedBaudRate = value;
			updateLinkColor();
		}
		
		[Event(name="Changed", type="flash.events.Event")]
		
		public static const CHANGE:String = "change";
		
		public function NetworkObjectLinker(nodeFrom : GenderObject, nodeTo : GenderObject, baudRate : Number = 0, propDelay : Number = 0, mtu : Number = 0)
		{
			this.nodeFrom = nodeFrom;
			this.nodeTo = nodeTo;
			
			BaudRate = baudRate;
			PropDelay = propDelay;
			MTU = mtu;
			
			super(nodeFrom.x + nodeFrom.width/2,
				nodeFrom.y + nodeFrom.height/2,
				nodeTo.x + nodeTo.width/2,
				nodeTo.y + nodeTo.height/2);
				
			UpdateID();
		}
		
		/**
		 * Update the link name based on the objects it connects
		 */
		public function UpdateID() : void
		{
			if (nodeFrom is RouterObject)
			{
				if (nodeTo is RouterObject)
					this.id = nodeFrom.nodeName+"_"+nodeTo.nodeName+"_link";
				else
					this.id = nodeTo.nodeName+"_link";
			} 
			else
				this.id = nodeFrom.nodeName+"_link";
			dispatchEvent(new Event(CHANGE));
		}
		
		/**
		 * Change the link color based on the workload overload
		 */
		private function updateLinkColor() : void
		{
			//FIXME: this should be taken from RGB colors (not names) and from "gradient" like class
			if (line == null) return;
			var _lineStroke : SolidStroke = new SolidStroke();
            _lineStroke.color = "Gray";
            var percent : Number = 1- UsedBaudRate/BaudRate;
            if (BaudRate == 0) null;
            else if (percent < 0.2) _lineStroke.color = "Red";
            else if (percent < 0.4) _lineStroke.color = "Orange";
            else if (percent < 0.6) _lineStroke.color = "Yellow";
            else if (percent < 0.8) _lineStroke.color = "LightGreen";
            else _lineStroke.color = "Green";
            _lineStroke.weight = 5;
            line.stroke = _lineStroke;
		}
		
		/**
		 * Redraw line
		 */
		public override function refresh() : void
		{
			updateLinkColor();
			line.x = nodeFrom.x + nodeFrom.width/2;
			line.y = nodeFrom.y + nodeFrom.height/2;
			line.x1 = nodeTo.x + nodeTo.width/2;
			line.y1 = nodeTo.y + nodeTo.height/2;
			this.draw(null,null);
		}

	}
}
