package gssim.gssimstudio.editor.gssim.workload
{
	import gssim.gssimstudio.editor.gssim.errors.ValidableObject;
	import gssim.gssimstudio.editor.gssim.xml.XMLBasicWrapper;
	
	import mx.collections.ArrayCollection;

	/**
	 * Class describing multi-distribution value
	 */
	public class MultiDistribution extends XMLBasicWrapper implements ValidableObject
	{
		private var distributions : ArrayCollection;
		
		public function get Distributions() : ArrayCollection
		{
			return distributions;
		}
		
		public function MultiDistribution()
		{
			distributions = new ArrayCollection();
		}

		/**
		 * Add new partial distribution
		 */
		public function addDist(dist : Dist, prob : Number) : void
		{
			dist.Probability = prob;
			distributions.addItem(dist);
		}
		
		/**
		 * Remove distribution at given index
		 */
		public function removeDist(distIndex : Number) : void
		{
			distributions.removeItemAt(distIndex);
		}
		
		/**
		 * Get distribution at given index
		 */
		public function getDist(distIndex : Number) : Dist
		{
			return distributions.getItemAt(distIndex) as Dist;
		}
		
		/**
		 * Update distribution at given index with probability value
		 */
		public function updateDist(distIndex : Number, prob : Number) : void
		{
			distributions.getItemAt(distIndex).Probability = prob;
		}

		public override function getXML(addLabel : Boolean) : XML
		{
			var xml : XML = <MultiDistribution />;
			if (addLabel) addStringAttribute(xml, "label", xml.toXMLString(), false);
			for each (var dist : Dist in distributions)
			{
				xml.appendChild(dist.getXML(addLabel));
			}
			return xml;
		}
		
		/**
		 * Clone this multi-distribution object
		 */
		public function clone() : MultiDistribution
		{
			var multi : MultiDistribution = new MultiDistribution();
			for each (var dist : Dist in distributions)
			{
				multi.addDist(dist.clone(), dist.Probability);
			}
			return multi;
		}
		
		public function validate() : ArrayCollection
		{
			var ac : ArrayCollection = new ArrayCollection();
			
			return ac;
		}
	}
}
