package gssim.gssimstudio.editor.gssim.workload
{
	import gssim.gssimstudio.editor.gssim.errors.AbstractError;
	import gssim.gssimstudio.editor.gssim.errors.ValidableObject;
	import gssim.gssimstudio.editor.gssim.errors.workload.PreferenceError;
	import gssim.gssimstudio.editor.gssim.workload.types.PreferenceType;
	import gssim.gssimstudio.editor.gssim.xml.XMLBasicWrapper;
	
	import mx.collections.ArrayCollection;

	/**
	 * Class for storing experiment preferences
	 */
	public class Preference extends XMLBasicWrapper implements ValidableObject
	{
		/**
		 * Type of the preference
		 */
		[Bindable]
		public var preferenceType : String;
		/**
		 * Collection of preference parameters included with this preference
		 */
		[Bindable]
		public var preferenceParameters : ArrayCollection;
		
		public function Preference()
		{
			preferenceType = PreferenceType.PRIORITY;
			preferenceParameters = new ArrayCollection();
		}
		
		public override function getXML(addLabel : Boolean) : XML
		{
			var xml : XML = <Preferences />;
			if (addLabel) addStringAttribute(xml, "label", xml.toXMLString(), false);
			addStringAttribute(xml, "preferenceType", preferenceType, true);
			for each (var param : PreferenceParameter in preferenceParameters)
			{
				xml.appendChild(param.getXML(addLabel));
			}
			return xml;
		}
		
		public function validate() : ArrayCollection
		{
			var ac : ArrayCollection = new ArrayCollection();
			var ae : AbstractError;
			for each (var param : PreferenceParameter in preferenceParameters)
			{
				if (preferenceType == null) ac.addItem(new PreferenceError("MissingPreferenceTypeError", this));
				for each (ae in param.validate()) ac.addItem(ae);
			}
			return ac;
		}
	}
}
