package gssim.gssimstudio.editor.gssim.workload
{
	import gssim.gssimstudio.editor.gssim.errors.ValidableObject;
	import gssim.gssimstudio.editor.gssim.errors.workload.ExistingRandParamsError;
	import gssim.gssimstudio.editor.gssim.xml.XMLBasicWrapper;
	
	import mx.collections.ArrayCollection;

	/**
	 * Basic value type, described by probabilistic parameters, periodic valid values and multi-distribution
	 */
	public class RandParams extends XMLBasicWrapper implements ValidableObject
	{
		private var tagName : XML = <RandParams />;
		
		[Bindable]
		public var params : ParameterAttributes;
		
		[Bindable]
		public var periodicValidValues : ArrayCollection;
		
		public var multiDist : MultiDistribution;
		
		public function RandParams(tagName : String)
		{
			params = new ParameterAttributes(tagName);
			this.tagName = XML(tagName);
			periodicValidValues = new ArrayCollection();
		}

		public override function getXML(addLabel : Boolean):XML
		{
			var xml : XML = tagName.copy();
			if (addLabel) addStringAttribute(xml, "label", xml.toXMLString(), false);
			if (periodicValidValues.length != 0 && multiDist != null) throw Error("PeriodicValidValues and MultiDistribution cannot be set together!");
			for each (var periodic : PeriodicValidValues in periodicValidValues)
			{
				xml.appendChild(periodic.getXML(addLabel));
			}
			if (multiDist != null) xml.appendChild(multiDist.getXML(addLabel));
			if (params != null) params.addParamsToTag(xml);
			return xml;
		}
		
		public function validate() : ArrayCollection
		{
			var ac : ArrayCollection = new ArrayCollection();
			if (periodicValidValues.length != 0 && multiDist != null) ac.addItem(new ExistingRandParamsError("CannotBeSetTogetherError", "PeriodicValidValues, MultiDistribution", this));
			return ac;
		}
		
	}
}
