package gssim.gssimstudio.editor.gssim.workload.file
{
	import flash.events.Event;
	import flash.net.FileReference;
	import flash.utils.ByteArray;
	
	import gssim.gssimstudio.editor.gssim.workload.ComputingResourceHostParameter;
	import gssim.gssimstudio.editor.gssim.workload.Dist;
	import gssim.gssimstudio.editor.gssim.workload.MultiDistribution;
	import gssim.gssimstudio.editor.gssim.workload.PeriodicValidValues;
	import gssim.gssimstudio.editor.gssim.workload.Preference;
	import gssim.gssimstudio.editor.gssim.workload.PreferenceParameter;
	import gssim.gssimstudio.editor.gssim.workload.RandParams;
	import gssim.gssimstudio.editor.gssim.workload.WorkloadBasic;
	
	import gssim.gssimstudio.editor.script.utils.DateTimeUtil;
	
	import gssim.gssimstudio.editor.views.WorkloadEditor;
	
	/**
	 * Loads workload configuration from file
	 */
	public class workloadReader
	{
		private var fref : FileReference;
				
		public function workloadReader()
		{
			fref = new FileReference();
			fref.addEventListener(Event.SELECT, selectHandler);
			fref.addEventListener(Event.COMPLETE, completeHandler);
			fref.browse();
		}
		
		public function selectHandler(event:Event):void
		{
		    fref.load();
		}
			
		private function readRandParam(xml : XML) : RandParams
		{
			var params : RandParams = new RandParams("<" + xml.localName() + "/>");
			//read attributes
			for each (var paramName : XML in xml.attributes())
			{
				if (!isNaN(parseFloat(paramName)))
					params.params.setProperty(paramName.name(), Number(paramName));
				else
					params.params.setProperty(paramName.name(), paramName);
			}
			if (xml.hasOwnProperty("MultiDistribution"))
			{
				params.multiDist = new MultiDistribution();
				for each (var distXML : XML in xml.MultiDistribution.dist)
				{
					var dist : Dist = new Dist("");
					//read attributes
					for each (var distParamName : XML in distXML.attributes())
					{
						if (!isNaN(parseFloat(distParamName)))
							dist.setProperty(distParamName.name(), Number(distParamName));
						else
							dist.setProperty(distParamName.name(), distParamName);
					}
					dist.Probability = Number(distXML);
					params.multiDist.addDist(dist, dist.Probability); 
				}
			}
			if (xml.hasOwnProperty("PeriodicValidValues"))
			{
				for each (var periodicXML : XML in xml.PeriodicValidValues)
				{
					var periodic : PeriodicValidValues = new PeriodicValidValues();
					//read attributes
					for each (var periodicParamName : XML in periodicXML.attributes())
					{
						if (!isNaN(parseFloat(periodicParamName)))
							periodic.params.setProperty(periodicParamName.name(), Number(periodicParamName));
						else
							periodic.params.setProperty(periodicParamName.name(), periodicParamName);
					}
					if (periodicXML.hasOwnProperty("BeginValidTime")) periodic.BeginValidTime = DateTimeUtil.convertFromStringToDate(periodicXML.BeginValidTime.toString());
					if (periodicXML.hasOwnProperty("EndValidTime")) periodic.EndValidTime = DateTimeUtil.convertFromStringToDate(periodicXML.EndValidTime.toString());
					params.periodicValidValues.addItem(periodic);
				}
			}
			return params;
		}
			
		public function completeHandler(event:Event):void
		{
			var workload : WorkloadBasic = new WorkloadBasic();
		    var data:ByteArray = fref.data;
			
			var xml : XML;
			xml = XML(data.readUTFBytes(data.bytesAvailable));
			
			for (var i : Number = 0; i < xml.children().length(); i++)
			{
				var inner_xml : XML = xml.children()[i];
				if (inner_xml.localName() == "SimulationStartTime")
					workload.setProperty(WorkloadBasic.SimulationStartTime, DateTimeUtil.convertFromStringToDate(inner_xml.toString()));
				else if (inner_xml.localName() == "SimulationTime")
					workload.setProperty(WorkloadBasic.SimulationTime, inner_xml.toString());
				else if (inner_xml.localName() == "ComputingResourceHostParameter")
				{
					var crhp : ComputingResourceHostParameter = new ComputingResourceHostParameter(inner_xml.@metric);
					if (inner_xml.hasOwnProperty("value")) crhp.value = readRandParam(inner_xml.value[0]);
					if (inner_xml.hasOwnProperty("range"))
					{
						if (inner_xml.range.hasOwnProperty("min")) crhp.range.min = readRandParam(inner_xml.range.min[0]);
						if (inner_xml.range.hasOwnProperty("max")) crhp.range.max = readRandParam(inner_xml.range.max[0]);
					}
					workload.CRHP.addItem(crhp);
				}
				else if (inner_xml.localName() == "Preferences")
				{
					var pref : Preference = new Preference();
					pref.preferenceType = inner_xml.@preferenceType;
					for each (var paramXML : XML in inner_xml.parameter)
					{
						var param : PreferenceParameter = new PreferenceParameter();
						param.Name = paramXML.name;
						param.OptimizationType = paramXML.optimizationType;
						if (paramXML.hasOwnProperty("endpoint")) param.Endpoint = paramXML.endpoint;
						param.Importance = readRandParam(paramXML.importance[0]);
						param.Value = readRandParam(paramXML.value[0]);
						pref.preferenceParameters.addItem(param);
					}
					workload.Prefs.addItem(pref);
				}
				else if (inner_xml.localName() == "ExecutionTime")
				{
					for (var j : Number = 0; j < inner_xml.children().length(); j++)
					{
						var inner_inner_xml : XML = inner_xml.children()[j];
						workload.setProperty("<"+inner_inner_xml.localName() + "/>", readRandParam(inner_inner_xml));
					}
				}
				else
				{
					workload.setProperty("<"+inner_xml.localName() + "/>", readRandParam(inner_xml));
				}
			}
			
			fref = null;
			
			WorkloadEditor.workload = workload;
		}

	}
}
