package gssim.gssimstudio.editor.gssim.xml
{
	import gssim.gssimstudio.editor.script.utils.DateTimeUtil;
	
	/**
	 * Basic XML handling class 
	 */
	public class XMLBasicWrapper implements XMLWrapper
	{
		public function XMLBasicWrapper()
		{
		}

		public function toXMLString():String
		{
			var xml : XML = getXML(false);
			if (xml == null) throw Error("XML was not created!");
			return xml.toXMLString();
		}
		
		public function getXML(addLabel : Boolean):XML
		{
			return null;
		}
		
		/**
		 * Append given xml with text property
		 */
		protected function appendChildWithProperty(parent : XML, properties : Object, propertyName : String, required : Boolean, addLabel : Boolean) : XML
		{
			return appendChildWithText(parent, propertyName, properties[propertyName], required, addLabel);
		}
		
		/**
		 * Append given xml with text property
		 */
		protected function appendChildWithText(parent : XML, propertyName : String, propertyValue : Object, required : Boolean, addLabel : Boolean) : XML
		{
			if (propertyValue != null)
			{
				var xml : XML = XML(propertyName);
				xml.appendChild(propertyValue);
				if (addLabel) addStringAttribute(xml, "label", propertyName, false); 
				parent.appendChild(xml);
				
			}
			else if (required) throw Error("Missing required element: " + propertyName);
			return parent;
		}
		
		/**
		 * Append given xml with the date value
		 */
		protected function appendChildWithDate(parent : XML, properties : Object, propertyName : String, required : Boolean, addLabel : Boolean) : XML
		{
			if (properties[propertyName] != null)
			{
				var xml : XML = XML(propertyName);
				xml.appendChild(DateTimeUtil.convertFromDateToString(properties[propertyName]));
				if (addLabel) addStringAttribute(xml, "label", propertyName, false); 
				parent.appendChild(xml);
				
			}
			else if (required) throw Error("Missing required element: " + propertyName);
			return parent;
		}
		
		/**
		 * Append child with xml
		 */
		protected function appendChildWithXML(parent : XML, properties : Object, propertyName : String, required : Boolean, addLabel : Boolean) : XML
		{
			if (properties[propertyName] != null) parent.appendChild(properties[propertyName].getXML(addLabel));
			else if (required) throw Error("Missing required element: " + propertyName);
			return parent;
		}
		
		/**
		 * Add attribute from number to xml tag
		 */
		protected function addNumberAttribute(parent : XML, attributeName : String, value : Number, required : Boolean) : XML
		{
			if (!isNaN(value)) parent.@[attributeName] = value;
			else if (required) throw Error("Missing required attribute: " + attributeName);
			return parent;
		}
		
		/**
		 * Add attribute from string to xml tag
		 */
		protected function addStringAttribute(parent : XML, attributeName : String, value : String, required : Boolean) : XML
		{
			if (value != null && value != "") parent.@[attributeName] = value;
			else if (required) throw Error("Missing required attribute: " + attributeName);
			return parent;
		}
		
	}
}
